@@ -4,7 +4,7 @@
 # This module is part of the desktop management solution opsi
 # (open pc server integration) http://www.opsi.org
 
-# Copyright (C) 2010-2014 uib GmbH <info@uib.de>
+# Copyright (C) 2010-2015 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -26,32 +26,31 @@ This backend executes the calls on a remote backend via JSONRPC.
 :copyright: uib GmbH <info@uib.de>
 :author: Jan Schneider <j.schneider@uib.de>
 :author: Niko Wenselowski <n.wenselowski@uib.de>
+:author: Erol Ueluekmen <e.ueluekmen@uib.de>
 :license: GNU Affero General Public License version 3
 """
 
-__version__ = '4.0.6.1'
-
 import base64
 import json
 import new
 import socket
 import time
-import types
 import threading
-import zlib
+from hashlib import md5
 from Queue import Queue, Empty
 from twisted.conch.ssh import keys
 from sys import version_info
 
-from OPSI.Logger import Logger, LOG_INFO
+from OPSI.Logger import Logger, LOG_INFO, LOG_NONE
 from OPSI.Types import (forceBool, forceFilename, forceFloat, forceInt,
 						forceList, forceUnicode)
 from OPSI.Types import (OpsiAuthenticationError, OpsiServiceVerificationError,
 						OpsiTimeoutError)
 from OPSI.Backend.Backend import Backend, DeferredCall
 from OPSI.Util import serialize, deserialize
-from OPSI.Util.HTTP import urlsplit, getSharedConnectionPool
+from OPSI.Util.HTTP import urlsplit, getSharedConnectionPool, deflateEncode, deflateDecode, gzipDecode
 
+__version__ = '4.0.7.12'
 
 logger = Logger()
 
@@ -73,8 +72,8 @@ class JSONRPC(DeferredCall):
 
 	def getRpc(self):
 		if self.jsonrpcBackend.isLegacyOpsi():
-			for i in xrange(len(self.params)):
-				if self.params[i] == '__UNDEF__':
+			for i, param in enumerate(self.params):
+				if param == '__UNDEF__':
 					self.params[i] = None
 
 		return {
@@ -86,11 +85,12 @@ class JSONRPC(DeferredCall):
 	def processResult(self, result):
 		try:
 			if result.get('error'):
+				logger.debug('Result from RPC contained error!')
 				error = result.get('error')
 				# Error occurred
-				if type(error) is dict and error.get('message'):
+				if isinstance(error, dict) and error.get('message'):
 					message = error['message']
-					exception = Exception(message)
+
 					try:
 						exceptionClass = eval(error.get('class', 'Exception'))
 						index = message.find(':')
@@ -98,27 +98,33 @@ class JSONRPC(DeferredCall):
 							message = message[index + 1:].lstrip()
 						exception = exceptionClass(u'%s (error on server)' % message)
 					except Exception:
-						pass
+						exception = Exception(message)
+
 					raise exception
-				raise Exception(u'%s (error on server)' % error)
-			self.result = deserialize(result.get('result'), preventObjectCreation=self.method.endswith('_getHashes'))
-		except Exception as e:
-			logger.logException(e)
-			self.error = e
-		self._gotResult()
+
+				raise Exception(u'{0} (error on server)'.format(error))
+			self.result = deserialize(
+				result.get('result'),
+				preventObjectCreation=self.method.endswith('_getHashes')
+			)
+		except Exception as error:
+			logger.logException(error)
+			self.error = error
 
 	def process(self):
-		try:
-			logger.debug(u"Executing jsonrpc method '%s' on host %s" % (self.method, self.jsonrpcBackend._host))
+		logger.debug(u"Executing jsonrpc method {0!r} on host {1!r}", self.method, self.jsonrpcBackend._host)
 
+		try:
 			rpc = json.dumps(self.getRpc())
-			logger.debug2(u"jsonrpc: %s" % rpc)
+			logger.debug2(u"jsonrpc: {0!r}", rpc)
+
 			response = self.jsonrpcBackend._request(baseUrl=self.baseUrl, data=rpc, retry=self.retry)
 			self.processResult(json.loads(response))
-		except Exception as e:
-			if not self.method in ('backend_exit', 'exit'):
-				logger.logException("Failed to process method '%s': %s" % (self.method, forceUnicode(e)), LOG_INFO)
-				self.error = e
+		except Exception as error:
+			if self.method not in ('backend_exit', 'exit'):
+				logger.logException("Failed to process method '%s': %s" % (self.method, forceUnicode(error)), LOG_INFO)
+				self.error = error
+		finally:
 			self._gotResult()
 
 
@@ -127,7 +133,8 @@ class JSONRPCThread(JSONRPC, threading.Thread):
 		if params is None:
 			params = []
 		threading.Thread.__init__(self)
-		JSONRPC.__init__(self,
+		JSONRPC.__init__(
+			self,
 			jsonrpcBackend=jsonrpcBackend,
 			baseUrl=baseUrl,
 			method=method,
@@ -218,28 +225,32 @@ class RpcQueue(threading.Thread):
 			logger.debug(u"Got response from host %s" % self.jsonrpcBackend._host)
 			try:
 				response = forceList(json.loads(response))
-			except Exception as e:
-				raise Exception(u"Failed to json decode response %s: %s" % (response, e))
+			except Exception as error:
+				raise Exception(u"Failed to json decode response %s: %s" % (response, error))
 
 			for resp in response:
 				try:
-					id = resp['id']
-				except Exception as e:
-					raise Exception(u"Failed to get id from: %s (%s): %s" % (resp, response, e))
+					responseId = resp['id']
+				except KeyError as error:
+					raise KeyError(u"Failed to get id from: %s (%s): %s" % (resp, response, error))
+
 				try:
-					jsonrpc = self.jsonrpcs[id]
-				except Exception as e:
-					raise Exception(u"Failed to get jsonrpc with id %s: %s" % (id, e))
+					jsonrpc = self.jsonrpcs[responseId]
+				except KeyError as error:
+					raise KeyError(u"Failed to get jsonrpc with id %s: %s" % (responseId, error))
+
 				try:
 					jsonrpc.processResult(resp)
-				except Exception as e:
-					raise Exception(u"Failed to process response %s with jsonrpc %s: %s" % (resp, jsonrpc, e))
-		except Exception as e:
+				except Exception as error:
+					raise Exception(u"Failed to process response %s with jsonrpc %s: %s" % (resp, jsonrpc, error))
+		except Exception as error:
 			if not isExit:
-				logger.logException(e)
+				logger.logException(error)
+
 			for jsonrpc in self.jsonrpcs.values():
-				jsonrpc.error = e
+				jsonrpc.error = error
 				jsonrpc._gotResult()
+
 		self.jsonrpcs = {}
 		self.idle.set()
 
@@ -279,6 +290,8 @@ class JSONRPCBackend(Backend):
 		self._verifyServerCert = False
 		self._verifyServerCertByCa = False
 		self._verifyByCaCertsFile = None
+		self._wrongHTTPHeaders = None
+		self._proxyURL = None
 
 		if not self._username:
 			self._username = u''
@@ -318,6 +331,9 @@ class JSONRPCBackend(Backend):
 				self._caCertFile = forceFilename(value)
 			elif option == 'verifyservercertbyca':
 				self._verifyServerCertByCa = forceBool(value)
+			elif option == 'proxyurl' and value is not None:
+			    logger.debug(u"ProxyURL detected: '%s'" % value)
+			    self._proxyURL = forceUnicode(value)
 
 		if not retry:
 			self._retryTime = 0
@@ -338,7 +354,8 @@ class JSONRPCBackend(Backend):
 			verifyServerCert=self._verifyServerCert,
 			serverCertFile=self._serverCertFile,
 			caCertFile=self._caCertFile,
-			verifyServerCertByCa=self._verifyServerCertByCa
+			verifyServerCertByCa=self._verifyServerCertByCa,
+			proxyURL=self._proxyURL
 		)
 
 		if self._connectOnInit:
@@ -374,7 +391,7 @@ class JSONRPCBackend(Backend):
 					res = self._jsonRPC('exit', retry=False)
 				else:
 					res = self._jsonRPC('backend_exit', retry=False)
-			except:
+			except Exception:
 				pass
 		if self._rpcQueue:
 			self._rpcQueue.stop()
@@ -402,76 +419,102 @@ class JSONRPCBackend(Backend):
 		if deflate and self.isLegacyOpsi():
 			logger.error(u"Refusing to set deflate because we are connected to legacy opsi service")
 			return
+
+		if deflate and self._wrongHTTPHeaders:
+			logger.error(u"Refusing to set deflate because opsi service answers with wrong HTTP header contents.")
+			return
+
 		self._deflate = deflate
 
+	def getDeflate(self):
+		return self._deflate
+
 	def isConnected(self):
 		return self._connected
 
 	def connect(self):
+		modules = None
+		realmodules = {}
+		mysqlBackend = False
+
 		async = self._async
 		self._async = False
+
+		if self._deflate:
+			logger.debug(u"Testing if deflated communication works...")
+			previousLogLevel = logger.getConsoleLevel()
+			logger.setConsoleLevel(LOG_NONE)
+			try:
+				self._jsonRPC(u'backend_info')
+				logger.debug(u"Deflated communication works!")
+			except Exception as error:
+				logger.setConsoleLevel(previousLogLevel)
+				logger.debug(u"Caught {0!r}", error)
+				logger.debug(u"Disabling deflate...")
+				self._deflate = False
+			finally:
+				logger.setConsoleLevel(previousLogLevel)
+
 		try:
-			modules = None
-			realmodules = {}
-			mysqlBackend = False
 			try:
 				self._interface = self._jsonRPC(u'backend_getInterface')
-				if (self._application.find('opsiclientd') != -1):
+				if 'opsiclientd' in self._application:
 					try:
-						modules = self._jsonRPC(u'backend_info').get('modules', None)
-						realmodules = self._jsonRPC(u'backend_info').get('realmodules', None)
+						backendInfo = self._jsonRPC(u'backend_info')
+						modules = backendInfo.get('modules', None)
+						realmodules = backendInfo.get('realmodules', None)
 						if modules:
 							logger.confidential(u"Modules: %s" % modules)
 						else:
 							modules = {'customer': None}
+
 						for m in self._interface:
-							if (m.get('name') == 'dispatcher_getConfig'):
+							if m.get('name') == 'dispatcher_getConfig':
 								for entry in self._jsonRPC(u'dispatcher_getConfig'):
 									for bn in entry[1]:
-										if (bn.lower().find("sql") != -1) and (len(entry[0]) <= 4) and (entry[0].find('*') != -1):
+										if "sql" in bn.lower() and len(entry[0]) <= 4 and '*' in entry[0]:
 											mysqlBackend = True
 								break
-					except Exception as e:
-						logger.info(forceUnicode(e))
+					except Exception as error:
+						logger.info(forceUnicode(error))
 			except (OpsiAuthenticationError, OpsiTimeoutError, OpsiServiceVerificationError, socket.error):
 				raise
-			except Exception as e:
-				logger.debug(u"backend_getInterface failed: %s, trying getPossibleMethods_listOfHashes" % forceUnicode(e))
+			except Exception as error:
+				logger.debug(u"backend_getInterface failed: {0}", forceUnicode(error))
+				logger.debug(u"trying getPossibleMethods_listOfHashes")
 				self._interface = self._jsonRPC(u'getPossibleMethods_listOfHashes')
 				logger.info(u"Legacy opsi")
 				self._legacyOpsi = True
 				self._deflate = False
 				self._jsonRPC(u'authenticated', retry=False)
+
 			if self._legacyOpsi:
 				self._createInstanceMethods34()
 			else:
 				self._createInstanceMethods(modules, realmodules, mysqlBackend)
+
 			self._connected = True
-			logger.info(u"%s: Connected to service" % self)
+			logger.info(u"{0}: Connected to service", self)
 		finally:
 			self._async = async
 
 	def _getRpcId(self):
-		self._rpcIdLock.acquire()
-		try:
+		with self._rpcIdLock:
 			self._rpcId += 1
-			if self._rpcId > 100000:
-				self._rpcId = 1
-		finally:
-			self._rpcIdLock.release()
+
 		return self._rpcId
 
 	def _processAddress(self, address):
 		self._protocol = 'https'
 		(scheme, host, port, baseurl, username, password) = urlsplit(address)
 		if scheme:
-			if not scheme in ('http', 'https'):
+			if scheme not in ('http', 'https'):
 				raise Exception(u"Protocol %s not supported" % scheme)
 			self._protocol = scheme
 		self._host = host
 		if port:
 			self._port = port
-		elif (self._protocol == 'https'):
+		elif self._protocol == 'https':
 			self._port = self._defaultHttpsPort
 		else:
 			self._port = self._defaultHttpPort
@@ -498,25 +541,26 @@ class JSONRPCBackend(Backend):
 		for method in self._interface:
 			# Create instance method
 			params = ['self']
-			params.extend( method.get('params', []) )
+			params.extend(method.get('params', []))
 			paramsWithDefaults = list(params)
-			for i in range(len(params)):
-				if params[i].startswith('*'):
-					params[i] = params[i][1:]
-					paramsWithDefaults[i] = params[i] + '="__UNDEF__"'
+			for index, param in enumerate(params):
+				if param.startswith('*'):
+					newParameter = param[1:]
+					params[index] = newParameter
+					paramsWithDefaults[index] = '{0}="__UNDEF__"'.format(newParameter)
 
 			logger.debug2("Creating instance method '%s'" % method['name'])
 
 			if len(params) == 2:
-				logger.debug2('def %s(%s):\n  if type(%s) == list: %s = [ %s ]\n  return self._jsonRPC(method = "%s", params = [%s])'\
-					% (method['name'], ', '.join(paramsWithDefaults), params[1], params[1], params[1], method['name'], ', '.join(params[1:])) )
-				exec 'def %s(%s):\n  if type(%s) == list: %s = [ %s ]\n  return self._jsonRPC(method = "%s", params = [%s])'\
-					% (method['name'], ', '.join(paramsWithDefaults), params[1], params[1], params[1], method['name'], ', '.join(params[1:]))
+				methodCode = ('def %s(%s):\n  if type(%s) == list: %s = [ %s ]\n  return self._jsonRPC(method = "%s", params = [%s])'
+					% (method['name'], ', '.join(paramsWithDefaults), params[1], params[1], params[1], method['name'], ', '.join(params[1:])))
+				logger.debug2(methodCode)
+				exec(methodCode)
 			else:
-				logger.debug2('def %s(%s): return self._jsonRPC(method = "%s", params = [%s])'\
-					% (method['name'], ', '.join(paramsWithDefaults), method['name'], ', '.join(params[1:])) )
-				exec 'def %s(%s): return self._jsonRPC(method = "%s", params = [%s])'\
-					% (method['name'], ', '.join(paramsWithDefaults), method['name'], ', '.join(params[1:]))
+				methodCode = ('def %s(%s): return self._jsonRPC(method = "%s", params = [%s])'
+					% (method['name'], ', '.join(paramsWithDefaults), method['name'], ', '.join(params[1:])))
+				logger.debug2(methodCode)
+				exec(methodCode)
 
 			setattr(self.__class__, method['name'], new.instancemethod(eval(method['name']), None, self.__class__))
 
@@ -540,7 +584,7 @@ class JSONRPCBackend(Backend):
 					raise Exception(u"MySQL backend in use but not licensed")
 			else:
 				logger.info(u"Verifying modules file signature")
-				publicKey = keys.Key.fromString(data = base64.decodestring('AAAAB3NzaC1yc2EAAAADAQABAAABAQCAD/I79Jd0eKwwfuVwh5B2z+S8aV0C5suItJa18RrYip+d4P0ogzqoCfOoVWtDojY96FDYv+2d73LsoOckHCnuh55GA0mtuVMWdXNZIE8Avt/RzbEoYGo/H0weuga7I8PuQNC/nyS8w3W8TH4pt+ZCjZZoX8S+IizWCYwfqYoYTMLgB0i+6TCAfJj3mNgCrDZkQ24+rOFS4a8RrjamEz/b81noWl9IntllK1hySkR+LbulfTGALHgHkDUlk0OSu+zBPw/hcDSOMiDQvvHfmR4quGyLPbQ2FOVm1TzE0bQPR+Bhx4V8Eo2kNYstG2eJELrz7J1TJI0rCjpB+FQjYPsP')).keyObject
+				publicKey = keys.Key.fromString(data=base64.decodestring('AAAAB3NzaC1yc2EAAAADAQABAAABAQCAD/I79Jd0eKwwfuVwh5B2z+S8aV0C5suItJa18RrYip+d4P0ogzqoCfOoVWtDojY96FDYv+2d73LsoOckHCnuh55GA0mtuVMWdXNZIE8Avt/RzbEoYGo/H0weuga7I8PuQNC/nyS8w3W8TH4pt+ZCjZZoX8S+IizWCYwfqYoYTMLgB0i+6TCAfJj3mNgCrDZkQ24+rOFS4a8RrjamEz/b81noWl9IntllK1hySkR+LbulfTGALHgHkDUlk0OSu+zBPw/hcDSOMiDQvvHfmR4quGyLPbQ2FOVm1TzE0bQPR+Bhx4V8Eo2kNYstG2eJELrz7J1TJI0rCjpB+FQjYPsP')).keyObject
 				data = u''
 				mks = modules.keys()
 				mks.sort()
@@ -548,22 +592,18 @@ class JSONRPCBackend(Backend):
 					if module in ('valid', 'signature'):
 						continue
 
-					if realmodules.has_key(module):
+					if module in realmodules:
 						val = realmodules[module]
-						if module == 'mysql':
-							if int(val) + 50 <= hostCount:
-								raise Exception(u"UNDERLICENSED: You have more Clients then licensed in modules file.  Disabling module:")
-							elif int(val) <= hostCount:
-								logger.warning("WARNING UNDERLICENSED: You have more Clients then licensed in modules file.")
-						else:
-							if int(val) > 0:
-								modules[module] = True
+						if int(val) > 0:
+							modules[module] = True
 					else:
 						val = modules[module]
-						if (val == False): val = 'no'
-						if (val == True):  val = 'yes'
+						if val is False:
+							val = 'no'
+						if val is True:
+							val = 'yes'
 					data += u'%s = %s\r\n' % (module.lower().strip(), val)
-				if not bool(publicKey.verify(md5(data).digest(), [ long(modules['signature']) ])):
+				if not bool(publicKey.verify(md5(data).digest(), [long(modules['signature'])])):
 					logger.error(u"Disabling mysql backend and license management module: modules file invalid")
 					if mysqlBackend:
 						raise Exception(u"MySQL backend in use but not licensed")
@@ -579,60 +619,54 @@ class JSONRPCBackend(Backend):
 		for method in self._interface:
 			try:
 				methodName = method['name']
+
+				if methodName in ('backend_exit', 'backend_getInterface'):
+					continue
+
 				args = method['args']
 				varargs = method['varargs']
 				keywords = method['keywords']
 				defaults = method['defaults']
 
-				if methodName in ('backend_exit', 'backend_getInterface'):
-					continue
-
-				argString = u''
-				callString = u''
-				for i in range(len(args)):
-					if args[i] == 'self':
+				argString = []
+				callString = []
+				for i, argument in enumerate(args):
+					if argument == 'self':
 						continue
-					if argString:
-						argString += u', '
-					if callString:
-						callString += u', '
-					argString += args[i]
-					callString += args[i]
-					if type(defaults) in (tuple, list) and (len(defaults) + i >= len(args)):
-						default = defaults[len(defaults)-len(args)+i]
+
+					if isinstance(defaults, (tuple, list)) and len(defaults) + i >= len(args):
+						default = defaults[len(defaults) - len(args) + i]
 						# TODO: watch out for Python 3
-						if type(default) is str:
-							default = u"'%s'" % default
-						elif type(default) is unicode:
-							default = u"u'%s'" % default
-						argString += u'=%s' % unicode(default)
+						if isinstance(default, (str, unicode)):
+							default = u"{0!r}".format(default).replace('"', "'")
+						argString.append(u'{0}={1}'.format(argument, unicode(default)))
+					else:
+						argString.append(argument)
+					callString.append(argument)
+
 				if varargs:
 					for vararg in varargs:
-						if argString:
-							argString += u', '
-						if callString:
-							callString += u', '
-						argString += u'*%s' % vararg
-						callString += vararg
-				if keywords:
-					if argString:
-						argString += u', '
-					if callString:
-						callString += u', '
-					argString += u'**%s' % keywords
-					callString += keywords
+						argString.append(u'*{0}'.format(vararg))
+						callString.append(vararg)
 
-				logger.debug2(u"Arg string is: %s" % argString)
-				logger.debug2(u"Call string is: %s" % callString)
+				if keywords:
+					argString.append(u'**{0}'.format(keywords))
+					callString.append(keywords)
+
+				argString = u', '.join(argString)
+				callString = u', '.join(callString)
+
+				logger.debug2(u"{1}: arg string is: {0!r}", argString, methodName)
+				logger.debug2(u"{1}: call string is: {0!r}", callString, methodName)
 				# This would result in not overwriting Backend methods like log_read, log_write, ...
 				# if getattr(self, methodName, None) is None:
-				if not licenseManagementModule and (methodName.find("license") != -1):
+				if not licenseManagementModule and "license" in methodName:
 					exec(u'def %s(self, %s): return' % (methodName, argString))
 				else:
 					exec(u'def %s(self, %s): return self._jsonRPC("%s", [%s])' % (methodName, argString, methodName, callString))
 				setattr(self, methodName, new.instancemethod(eval(methodName), self, self.__class__))
-			except Exception as e:
-				logger.critical(u"Failed to create instance method '%s': %s" % (method, e))
+			except Exception as error:
+				logger.critical(u"Failed to create instance method '%s': %s" % (method, error))
 
 	def _jsonRPC(self, method, params=[], retry=True):
 		if self._async:
@@ -647,56 +681,73 @@ class JSONRPCBackend(Backend):
 	def _request(self, baseUrl, data, retry=True):
 		headers = {
 			'user-agent': self._application,
-			'Accept': 'application/json-rpc, text/plain'
+			'Accept': 'application/json, text/plain',
+			'Accept-Encoding': 'deflate, gzip',
+			'content-type': 'application/json',
 		}
-		if type(data) is types.StringType:
+		if isinstance(data, str):
 			data = unicode(data, 'utf-8')
 		data = data.encode('utf-8')
 
-		logger.debug2(u"Request to host '%s', baseUrl: %s, query '%s'" % (self._host, baseUrl, data))
+		logger.debug2(u"Request to host {0!r}, baseUrl: {1!r}, query: {2!r}".format(self._host, baseUrl, data))
 
 		if self._deflate:
 			logger.debug2(u"Compressing data")
-			headers['Accept'] += ', gzip-application/json-rpc'
-			headers['content-type'] = 'gzip-application/json-rpc'
-			headers['Accept-Encoding'] = 'gzip'
-			headers['Content-Encoding'] = 'gzip'
-			level = 1
-			data = zlib.compress(data, level)
+			headers['Content-Encoding'] = 'deflate'
+
+			data = deflateEncode(data)
 			# Fix for python 2.7
 			# http://bugs.python.org/issue12398
-			if (version_info >= (2,7)):
+			if version_info >= (2, 7):
 				data = bytearray(data)
-		else:
-			headers['content-type'] = 'application/json-rpc'
+			logger.debug2(u"Data compressed.")
 
 		headers['content-length'] = len(data)
 
 		auth = (self._username + u':' + self._password).encode('latin-1')
-		headers['Authorization'] = 'Basic ' + base64.encodestring(auth).strip()
+		headers['Authorization'] = 'Basic ' + base64.b64encode(auth)
 
 		if self._sessionId:
 			headers['Cookie'] = self._sessionId
 
+		logger.debug("Posting request...")
 		response = self._connectionPool.urlopen(method='POST', url=baseUrl, body=data, headers=headers, retry=retry)
 
+		return self._processResponse(response)
+
+	def _processResponse(self, response):
+		logger.debug2("Processing response...")
 		# Get cookie from header
 		cookie = response.getheader('set-cookie', None)
 		if cookie:
 			# Store sessionId cookie
 			sessionId = cookie.split(';')[0].strip()
-			if (sessionId != self._sessionId):
+			if sessionId != self._sessionId:
 				self._sessionId = sessionId
 
 		contentType = response.getheader('content-type', '')
 		contentEncoding = response.getheader('content-encoding', '').lower()
-		logger.debug(u"Content-Type: %s, Content-Encoding: %s" % (contentType, contentEncoding))
+		logger.debug(u"Content-Type: {0}, Content-Encoding: {1}", contentType, contentEncoding)
 
 		response = response.data
-		if (contentEncoding == 'gzip') or contentType.lower().startswith('gzip'):
-			logger.debug(u"Expecting compressed data from server")
-			response = zlib.decompress(response)
-		logger.debug2(response)
+		if contentType.lower().startswith('gzip'):
+			# To stay compatible with old versions of the opsiconfd
+			# we try to decompress the response with deflate even
+			# though gzip was stated.
+			# Content-type was usually gzip-application/json
+			logger.debug(u"Expecting deflated data from server (backwards compatible)")
+			response = deflateDecode(response)
+
+			if self._wrongHTTPHeaders is None:
+				self._wrongHTTPHeaders = True
+		elif contentEncoding == 'gzip':
+			logger.debug(u"Expecting gzip'ed data from server")
+			response = gzipDecode(response)
+		elif contentEncoding == "deflate":
+			logger.debug(u"Expecting deflated data from server")
+			response = deflateDecode(response)
+
+		logger.debug2(u"Response is: {0}", response)
 		return response
 
 	def getInterface(self):
@@ -704,3 +755,9 @@ class JSONRPCBackend(Backend):
 
 	def backend_getInterface(self):
 		return self._interface
+
+	def __repr__(self):
+		if self._name:
+			return u'<{0}(address={1!r}, host={2!r}, deflate={3!r})>'.format(self.__class__.__name__, self._name, self._host, self._deflate)
+		else:
+			return u'<{0}(host={1!r}, deflate={2!r})>'.format(self.__class__.__name__, self._host, self._deflate)
