@@ -35,7 +35,7 @@ from OPSI.Types import forceBool, forceHostId, forceList
 from OPSI.Util.Message import ProgressSubject
 
 
-__version__ = '4.0.6.13'
+__version__ = '4.0.6.48'
 
 logger = Logger()
 
@@ -159,6 +159,8 @@ class BackendReplicator(object):
 			end = self._getNumberOfObjectClassesToProcess(audit, license)
 			if self.__cleanupFirst:
 				end += 1
+			if self.__newServerId:
+				end += 1
 			self.__overallProgressSubject.setEnd(end)
 
 			if self.__cleanupFirst:
@@ -335,6 +337,10 @@ class BackendReplicator(object):
 				self.__overallProgressSubject.addToState(1)
 
 			if self.__newServerId:
+				self.__currentProgressSubject.reset()
+				self.__currentProgressSubject.setMessage(u"Renaming server")
+				self.__currentProgressSubject.setTitle(u"Renaming server")
+				self.__currentProgressSubject.setEnd(1)
 				if not self.__oldServerId:
 					if configServer:
 						self.__oldServerId = configServer.id
@@ -344,18 +350,29 @@ class BackendReplicator(object):
 						logger.error(u"No config/depot servers found")
 
 				if self.__oldServerId and self.__oldServerId != self.__newServerId:
-					logger.notice(u"Renaming config server '%s' to '%s'" % (self.__oldServerId, self.__newServerId))
-					wb.host_renameOpsiDepotserver(id=self.__oldServerId, newId=self.__newServerId)
+					logger.notice(u"Renaming config server {0!r} to {1!r}".format(self.__oldServerId, self.__newServerId))
+					renamingBackend = wb
+					try:
+						renamingBackend.host_renameOpsiDepotserver()
+					except TypeError:
+						pass  # Missing arguments but method exists
+					except AttributeError:
+						# Missing the method - need to use extended backend
+						renamingBackend = self._extendedWriteBackend
+
+					renamingBackend.host_renameOpsiDepotserver(id=self.__oldServerId, newId=self.__newServerId)
 
 					newDepots = []
-					for depot in wb.host_getObjects(type='OpsiDepotserver'):
+					for depot in renamingBackend.host_getObjects(type='OpsiDepotserver'):
 						hash = depot.toHash()
 						del hash['type']
 						if depot.id == self.__newServerId:
 							newDepots.append(OpsiConfigserver.fromHash(hash))
 						else:
 							newDepots.append(OpsiDepotserver.fromHash(hash))
-					wb.host_createObjects(newDepots)
+					renamingBackend.host_createObjects(newDepots)
+
+				self.__overallProgressSubject.addToState(1)
 		finally:
 			wb.backend_setOptions({'additionalReferentialIntegrityChecks': aric})
 
