@@ -1,39 +1,40 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+
+# This module is part of the desktop management solution opsi
+# (open pc server integration) http://www.opsi.org
+
+# Copyright (C) 2010-2016 uib GmbH
+
+# http://www.uib.de/
+
+# All rights reserved.
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-opsi python library - Worker
+Worker for the various interfaces.
 
-This module is part of the desktop management solution opsi
-(open pc server integration) http://www.opsi.org
-
-Copyright (C) 2010, 2013 uib GmbH
-
-http://www.uib.de/
-
-All rights reserved.
-
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
-
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-@copyright:	uib GmbH <info@uib.de>
-@author: Jan Schneider <j.schneider@uib.de>
-@author: Niko Wenselowski <n.wenselowski@uib.de>
-@license: GNU Affero General Public License version 3
+:copyright:	uib GmbH <info@uib.de>
+:author: Jan Schneider <j.schneider@uib.de>
+:author: Niko Wenselowski <n.wenselowski@uib.de>
+:license: GNU Affero General Public License version 3
 """
 
+import os.path
 import base64
 import urllib
-import zlib
+
 from twisted.internet import defer, reactor, threads
 from twisted.python import failure
 
@@ -43,6 +44,7 @@ from OPSI.Logger import Logger, LOG_ERROR, LOG_INFO
 from OPSI.Types import (forceUnicode, forceList, OpsiBadRpcError,
 						OpsiAuthenticationError)
 from OPSI.Util import objectToHtml, toJson, fromJson, serialize
+from OPSI.Util.HTTP import deflateEncode, deflateDecode, gzipEncode, gzipDecode
 from OPSI.Service.JsonRpc import JsonRpc
 
 logger = Logger()
@@ -233,20 +235,20 @@ interfacePage = u'''<?xml version="1.0" encoding="UTF-8"?>
 
 class WorkerOpsi:
 	def __init__(self, service, request, resource):
-		self.service   = service
+		self.service = service
 		if request.headers.hasHeader("x-forwarded-for"):
 			# overloading request because proxy detected
 			request.remoteAddr.host = request.headers.getRawHeaders("x-forwarded-for")[0]
-		self.request   = request
-		self.query     = u''
-		self.gzip      = False
-		self.path      = u''
-		self.resource  = resource
-		self.session   = None
+		self.request = request
+		self.query = u''
+		self.gzip = False
+		self.path = u''
+		self.resource = resource
+		self.session = None
 		self.authRealm = 'OPSI Service'
 
 	def process(self):
-		logger.info(u"Worker %s started processing" % self)
+		logger.info(u"Worker {0} started processing", self)
 		deferred = defer.Deferred()
 		deferred.addCallback(self._getSession)
 		deferred.addCallback(self._authenticate)
@@ -262,6 +264,7 @@ class WorkerOpsi:
 	def _getSessionHandler(self):
 		if hasattr(self.service, '_getSessionHandler'):
 			return self.service._getSessionHandler()
+
 		return None
 
 	def _delayResult(self, seconds, result):
@@ -273,28 +276,31 @@ class WorkerOpsi:
 
 			def returnResult(self):
 				self.deferred.callback(self.result)
+
 		return DelayResult(seconds, result).deferred
 
 	def _errback(self, failure):
-		logger.debug2("%s._errback" % self.__class__.__name__)
+		logger.debug2("{0}._errback", self.__class__.__name__)
 
 		self._freeSession(failure)
 
 		result = self._renderError(failure)
 		result = self._setCookie(result)
 		result.code = responsecode.INTERNAL_SERVER_ERROR
+
 		try:
 			failure.raiseException()
-		except OpsiAuthenticationError as e:
-			logger.logException(e)
+		except OpsiAuthenticationError as error:
+			logger.logException(error)
 			result.code = responsecode.UNAUTHORIZED
-			result.headers.setHeader('www-authenticate', [('basic', { 'realm': self.authRealm } )])
-		except OpsiBadRpcError as e:
-			logger.logException(e)
+			result.headers.setHeader('www-authenticate', [('basic', {'realm': self.authRealm})])
+		except OpsiBadRpcError as error:
+			logger.logException(error)
 			result.code = responsecode.BAD_REQUEST
-		except Exception as e:
-			logger.logException(e, LOG_ERROR)
+		except Exception as error:
+			logger.logException(error, LOG_ERROR)
 			logger.error(failure)
+
 		return result
 
 	def _renderError(self, failure):
@@ -303,97 +309,102 @@ class WorkerOpsi:
 		error = u'Unknown error'
 		try:
 			failure.raiseException()
-		except Exception as e:
-			error = forceUnicode(e)
+		except Exception as error:
+			error = forceUnicode(error)
 		result.stream = stream.IByteStream(stream.IByteStream(error.encode('utf-8')))
 		return result
 
 	def _freeSession(self, result):
 		if self.session:
-			logger.debug(u"Freeing session %s" % self.session)
+			logger.debug(u"Freeing session {0}", self.session)
 			self.session.decreaseUsageCount()
 		return result
 
 	def _getAuthorization(self):
-		(user, password) = (u'', u'')
+		user = password = u''
 		logger.debug(u"Trying to get username and password from Authorization header")
 		auth = self.request.headers.getHeader('Authorization')
 		if auth:
-			logger.debug(u"Authorization header found (type: %s)" % auth[0])
+			logger.debug(u"Authorization header found (type: {0})", auth[0])
 			try:
 				encoded = auth[1]
 
-				logger.confidential(u"Auth encoded: %s" % encoded)
+				logger.confidential(u"Auth encoded: {0}", encoded)
 				parts = unicode(base64.decodestring(encoded), 'latin-1').split(':')
-				if (len(parts) > 6):
+				if len(parts) > 6:
 					user = u':'.join(parts[:6])
 					password = u':'.join(parts[6:])
 				else:
 					user = parts[0]
 					password = u':'.join(parts[1:])
 				user = user.strip()
-				logger.confidential(u"Client supplied username '%s' and password '%s'" % (user, password))
-			except Exception as e:
-				logger.error(u"Bad Authorization header from '%s': %s" % (self.request.remoteAddr.host, e))
+				logger.confidential(u"Client supplied username {0!r} and password {1!r}", user, password)
+			except Exception as error:
+				logger.error(u"Bad Authorization header from '{0}': {1}", self.request.remoteAddr.host, error)
+
 		return (user, password)
 
 	def _getCredentials(self):
 		return self._getAuthorization()
 
 	def _getUserAgent(self):
-		# Get user agent
-		userAgent = None
 		try:
 			userAgent = self.request.headers.getHeader('user-agent')
 		except Exception:
 			logger.info(u"Client '%s' did not supply user-agent" % self.request.remoteAddr.host)
+			userAgent = None
+
 		if not userAgent:
 			userAgent = 'unknown'
+
 		return userAgent
 
 	def _getSessionId(self):
-		# Get session id from cookie request header
+		"Get session id from cookie request header"
 		sessionId = u''
 		try:
-			for (k, v) in self.request.headers.getAllRawHeaders():
-				if (k.lower() == 'cookie'):
-					for cookie in v:
+			for (headerTag, headerValue) in self.request.headers.getAllRawHeaders():
+				if headerTag.lower() == 'cookie':
+					for cookie in headerValue:
 						for c in cookie.split(';'):
-							if (c.find('=') == -1):
+							if '=' not in c:
 								continue
+
 							(name, value) = c.split('=', 1)
-							if (name.strip() == self._getSessionHandler().sessionName):
+							if name.strip() == self._getSessionHandler().sessionName:
 								sessionId = forceUnicode(value.strip())
 								break
+
 					break
-		except Exception as e:
-			logger.error(u"Failed to get cookie from header: %s" % e)
+		except Exception as error:
+			logger.error(u"Failed to get cookie from header: {0}", error)
+
 		return sessionId
 
 	def _getSession(self, result):
 		''' This method restores a session or generates a new one. '''
 		self.session = None
 
-		logger.confidential(u"Request headers: %s " % self.request.headers)
+		logger.confidential(u"Request headers: {0}", self.request.headers)
 
 		userAgent = self._getUserAgent()
-
-		# Get session handler
 		sessionHandler = self._getSessionHandler()
-
-		# Get session id from cookie request header
 		sessionId = self._getSessionId()
 
 		# Get Session object
 		self.session = sessionHandler.getSession(sessionId, self.request.remoteAddr.host)
-		if (sessionId == self.session.uid):
-			logger.info(u"Reusing session for client '%s', application '%s'" % (self.request.remoteAddr.host, userAgent))
+		if sessionId == self.session.uid:
+			logger.info(u"Reusing session for client '{0}', application '{1}'", self.request.remoteAddr.host, userAgent)
 		elif sessionId:
-			logger.notice(u"Application '%s' on client '%s' supplied non existing session id: %s" % (userAgent, self.request.remoteAddr.host, sessionId))
+			logger.notice(u"Application '{0}' on client '{1}' supplied non existing session id: {2}", userAgent, self.request.remoteAddr.host, sessionId)
 
 		if sessionHandler and self.session.ip and (self.session.ip != self.request.remoteAddr.host):
-			logger.critical(u"Client ip '%s' does not match session ip '%s', deleting old session and creating a new one" \
-				% (self.request.remoteAddr.host, self.session.ip) )
+			logger.critical(
+				u"Client ip '{0}' does not match session ip '{1}', "
+				u"deleting old session and creating a new one",
+				self.request.remoteAddr.host,
+				self.session.ip
+			)
 			sessionHandler.deleteSession(self.session.uid)
 			self.session = sessionHandler.getSession()
 
@@ -402,14 +413,22 @@ class WorkerOpsi:
 
 		# Set user-agent / application
 		if self.session.userAgent and (self.session.userAgent != userAgent):
-			logger.warning(u"Application changed from '%s' to '%s' for existing session of client '%s'" \
-				% (self.session.userAgent, userAgent, self.request.remoteAddr.host))
+			logger.warning(
+				u"Application changed from '{0}' to '{1}' for existing session of client '{2}'",
+				self.session.userAgent,
+				userAgent,
+				self.request.remoteAddr.host
+			)
 		self.session.userAgent = userAgent
 
-		logger.confidential(u"Session id is '%s' for client '%s', application '%s'" \
-			% (self.session.uid, self.request.remoteAddr.host, self.session.userAgent))
+		logger.confidential(
+			u"Session id is {0!r} for client {1!r}, application {2!r}",
+			self.session.uid,
+			self.request.remoteAddr.host,
+			self.session.userAgent
+		)
 
-		logger.confidential(u"Session content: %s" % self.session.__dict__)
+		logger.confidential(u"Session content: {0}", self.session.__dict__)
 		return result
 
 	def _setCookie(self, result):
@@ -421,12 +440,14 @@ class WorkerOpsi:
 		cookie = http_headers.Cookie(self.session.name.encode('ascii', 'replace'), self.session.uid.encode('ascii', 'replace'), path='/')
 		if not isinstance(result, http.Response):
 			result = http.Response()
-		result.headers.setHeader('set-cookie', [ cookie ] )
+		result.headers.setHeader('set-cookie', [cookie])
 		return result
 
 	def _authenticate(self, result):
-		''' This function tries to authenticate a user.
-		    Raises an exception on authentication failure. '''
+		'''
+		This function tries to authenticate a user.
+		Raises an exception on authentication failure.
+		'''
 		if self.session.authenticated:
 			return result
 
@@ -434,23 +455,25 @@ class WorkerOpsi:
 			# Get authorization
 			(self.session.user, self.session.password) = self._getCredentials()
 			self.session.authenticated = True
-		except Exception as e:
-			logger.logException(e, LOG_INFO)
+		except Exception as error:
+			logger.logException(error, LOG_INFO)
 			self._freeSession(result)
 			self._getSessionHandler().deleteSession(self.session.uid)
-			raise OpsiAuthenticationError(u"Forbidden: %s" % e)
+			raise OpsiAuthenticationError(u"Forbidden: %s" % error)
+
 		return result
 
 	def _getQuery(self, result):
 		self.query = ''
-		if   (self.request.method == 'GET'):
-			self.query = urllib.unquote( self.request.querystring )
-		elif (self.request.method == 'POST'):
+		if self.request.method == 'GET':
+			self.query = urllib.unquote(self.request.querystring)
+		elif self.request.method == 'POST':
 			# Returning deferred needed for chaining
 			d = stream.readStream(self.request.stream, self._handlePostData)
 			return d
 		else:
 			raise ValueError(u"Unhandled method '%s'" % self.request.method)
+
 		return result
 
 	def _handlePostData(self, chunk):
@@ -458,31 +481,52 @@ class WorkerOpsi:
 
 	def _decodeQuery(self, result):
 		try:
-			if (self.request.method == 'POST'):
+			if self.request.method == 'POST':
 				contentType = self.request.headers.getHeader('content-type')
-				contentEncoding = None
 				try:
 					contentEncoding = self.request.headers.getHeader('content-encoding')[0].lower()
-				except:
-					pass
-				logger.debug(u"Content-Type: %s, Content-Encoding: %s" % (contentType, contentEncoding))
-				if (contentEncoding == 'gzip') or (contentType and contentType.mediaType.startswith('gzip')):
-					logger.debug(u"Expecting compressed data from client")
-					self.query = zlib.decompress(self.query)
+				except Exception:
+					contentEncoding = None
+
+				logger.debug(u"Content-Type: {0}, Content-Encoding: {1}", contentType, contentEncoding)
+				if contentType and contentType.mediaType.startswith('gzip'):
+					# Invalid MIME type.
+					# Probably it is gzip-application/json-rpc and therefore
+					# we need to behave like we did before.
+					logger.debug(u"Expecting compressed data from client (backwards compatible)")
+					self.query = deflateDecode(self.query)
 					self.gzip = True
-			self.query = unicode(self.query, 'utf-8')
-		except (UnicodeError, UnicodeEncodeError):
-			self.query = unicode(self.query, 'utf-8', 'replace')
-		logger.debug2(u"query: %s" % self.query)
+				elif contentEncoding == 'gzip':
+					logger.debug(u"Expecting gzip compressed data from client")
+					self.query = gzipDecode(self.query)
+					self.gzip = True
+				elif contentEncoding == 'deflate':
+					logger.debug(u"Expecting deflate compressed data from client")
+					self.query = deflateDecode(self.query)
+					self.gzip = True
+
+			if not isinstance(self.query, unicode):
+				self.query = unicode(self.query, 'utf-8')
+		except (UnicodeError, UnicodeEncodeError) as error:
+			logger.logException(error)
+			if not isinstance(self.query, unicode):
+				self.query = unicode(self.query, 'utf-8', 'replace')
+		except Exception as error:
+			logger.logException(error)
+			logger.warning("Unexpected error during decoding of query: {0}", error)
+			raise error
+
+		logger.debug2(u"query: {0}", self.query)
 		return result
 
 	def _processQuery(self, result):
-		logger.warning(u"Class %s should overwrite _processQuery" % self.__class__.__name__)
+		logger.warning(u"Class {0} should overwrite _processQuery", self.__class__.__name__)
 		return self._decodeQuery(result)
 
 	def _generateResponse(self, result):
 		if not isinstance(result, http.Response):
 			result = http.Response()
+
 		result.code = responsecode.OK
 		result.headers.setHeader('content-type', http_headers.MimeType("text", "html", {"charset": "utf-8"}))
 		result.stream = stream.IByteStream("")
@@ -493,6 +537,9 @@ class WorkerOpsi:
 
 
 class WorkerOpsiJsonRpc(WorkerOpsi):
+
+	RFC_CONFORM_HEADERS = os.path.exists('/etc/opsi/opsi.header.fix.enable')
+
 	def __init__(self, service, request, resource):
 		WorkerOpsi.__init__(self, service, request, resource)
 
@@ -501,7 +548,7 @@ class WorkerOpsiJsonRpc(WorkerOpsi):
 		self._rpcs = []
 
 	def _getCallInstance(self, result):
-		logger.warning(u"Class %s should overwrite _getCallInstance" % self.__class__.__name__)
+		logger.warning(u"Class {0} should overwrite _getCallInstance", self.__class__.__name__)
 		self._callInstance = None
 		self._callInterface = None
 
@@ -513,16 +560,15 @@ class WorkerOpsiJsonRpc(WorkerOpsi):
 		if not self._callInterface:
 			raise Exception(u"Call interface not defined in %s" % self)
 
-		rpcs = []
 		try:
-			rpcs = fromJson(self.query, preventObjectCreation = True)
+			rpcs = fromJson(self.query, preventObjectCreation=True)
 			if not rpcs:
-				raise Exception(u"Got no rpcs")
+				raise ValueError(u"Got no rpcs")
 		except Exception as e:
 			raise OpsiBadRpcError(u"Failed to decode rpc: %s" % e)
 
 		for rpc in forceList(rpcs):
-			rpc = JsonRpc(instance = self._callInstance, interface = self._callInterface, rpc = rpc)
+			rpc = JsonRpc(instance=self._callInstance, interface=self._callInterface, rpc=rpc)
 			self._rpcs.append(rpc)
 
 		return result
@@ -550,50 +596,63 @@ class WorkerOpsiJsonRpc(WorkerOpsi):
 		return deferred
 
 	def _generateResponse(self, result):
-		if not isinstance(result, http.Response):
-			result = http.Response()
-		result.code = responsecode.OK
-
+		invalidMime = False  # For handling the invalid MIME type "gzip-application/json-rpc"
 		encoding = None
 		try:
-			if ('deflate' in self.request.headers.getHeader('Accept-Encoding')):
-				encoding = 'deflate'
-			if ('gzip' in self.request.headers.getHeader('Accept-Encoding')):
+			if 'gzip' in self.request.headers.getHeader('Accept-Encoding'):
 				encoding = 'gzip'
-		except Exception as e:
-			pass
-		if not encoding:
-			try:
-				if self.request.headers.getHeader('Accept'):
-					for accept in self.request.headers.getHeader('Accept').keys():
-						if accept.mediaType.startswith('gzip'):
-							encoding = 'gzip'
-							break
-			except Exception as e:
-				logger.error(u"Failed to get accepted mime types from header: %s" % e)
+			if 'deflate' in self.request.headers.getHeader('Accept-Encoding'):
+				encoding = 'deflate'
+		except Exception as error:
+			logger.debug2("Failed to get Accept-Encoding from request header: {0}".format(error))
 
-		if encoding:
-			result.headers.setHeader('content-type', http_headers.MimeType("gzip-application", "json", {"charset": "utf-8"}))
-			result.headers.setHeader('content-encoding', [encoding])
-		else:
-			result.headers.setHeader('content-type', http_headers.MimeType("application", "json", {"charset": "utf-8"}))
+		try:
+			if self.request.headers.getHeader('Accept'):
+				for accept in self.request.headers.getHeader('Accept').keys():
+					if accept.mediaType.startswith('gzip'):
+						invalidMime = True
+						encoding = 'gzip'
+						break
+		except Exception as error:
+			logger.error(u"Failed to get accepted mime types from header: {0}", error)
 
-		response = []
+		response = [serialize(rpc.getResponse()) for rpc in self._rpcs]
 
-		for rpc in self._rpcs:
-			response.append(serialize(rpc.getResponse()))
-		if (len(response) == 1):
+		if len(response) == 1:
 			response = response[0]
 		if not response:
 			response = None
 
-		if encoding:
-			# level 1 (fastest) to 9 (most compression)
-			level = 1
-			logger.debug(u"Sending compressed (level: %d) data" % level)
-			result.stream = stream.IByteStream(zlib.compress(toJson(response).encode('utf-8'), level))
+		if not isinstance(result, http.Response):
+			result = http.Response()
+		result.code = responsecode.OK
+
+		result.headers.setHeader('content-type', http_headers.MimeType("application", "json", {"charset": "utf-8"}))
+
+		if not self.RFC_CONFORM_HEADERS or invalidMime:
+			if encoding in ('deflate', 'gzip'):
+				# The invalid requests expect the encoding set to
+				# gzip but the content is deflated.
+				result.headers.setHeader('content-encoding', [encoding])
+				result.headers.setHeader('content-type', http_headers.MimeType("gzip-application", "json", {"charset": "utf-8"}))
+				logger.debug(u"Sending deflated data (backwards compatible - with content-encoding {0!r})", encoding)
+				result.stream = stream.IByteStream(deflateEncode(toJson(response).encode('utf-8')))
+			else:
+				logger.debug(u"Sending plain data")
+				result.stream = stream.IByteStream(toJson(response).encode('utf-8'))
+		elif encoding == "deflate":
+			result.headers.setHeader('content-encoding', [encoding])
+
+			logger.debug(u"Sending deflated data")
+			result.stream = stream.IByteStream(deflateEncode(toJson(response).encode('utf-8')))
+		elif encoding == "gzip":
+			result.headers.setHeader('content-encoding', [encoding])
+
+			logger.debug(u"Sending gzip compressed data")
+			result.stream = stream.IByteStream(gzipEncode(toJson(response).encode('utf-8')))
 		else:
 			result.stream = stream.IByteStream(toJson(response).encode('utf-8'))
+
 		return result
 
 	def _renderError(self, failure):
@@ -602,8 +661,8 @@ class WorkerOpsiJsonRpc(WorkerOpsi):
 		error = u'Unknown error'
 		try:
 			failure.raiseException()
-		except Exception as e:
-			error = {'class': e.__class__.__name__, 'message': unicode(e)}
+		except Exception as err:
+			error = {'class': err.__class__.__name__, 'message': unicode(err)}
 			error = toJson({"id": None, "result": None, "error": error})
 		result.stream = stream.IByteStream(error.encode('utf-8'))
 		return result
@@ -612,14 +671,11 @@ class WorkerOpsiJsonRpc(WorkerOpsi):
 class MultiprocessWorkerOpsiJsonRpc(WorkerOpsiJsonRpc):
 
 	def _processQuery(self, result):
-
 		logger.debug(u"Using multiprocessing to handle rpc.")
 
-
-
 		def cleanup(rpc):
-			if (rpc.getMethodName() == 'backend_exit'):
-				logger.notice(u"User '%s' asked to close the session" % self.session.user)
+			if rpc.getMethodName() == 'backend_exit':
+				logger.notice(u"User '{0}' asked to close the session", self.session.user)
 				self._freeSession(result)
 				self.service._getSessionHandler().deleteSession(self.session.uid)
 
@@ -630,17 +686,18 @@ class MultiprocessWorkerOpsiJsonRpc(WorkerOpsiJsonRpc):
 				cleanup(rpc)
 				self._addRpcToStatistics(None, rpc)
 				self.session.setLastRpcMethod(rpc.getMethodName())
+
 			return rpcs
 
 		def makeInstanceCall():
 			contentType = self.request.headers.getHeader('content-type')
-			contentEncoding = None
 			try:
 				contentEncoding = self.request.headers.getHeader('content-encoding')[0].lower()
-			except:
-				pass
-			d = self._callInstance.processQuery(self.query,
-				(contentEncoding == 'gzip') or (contentType and contentType.mediaType.startswith('gzip')))
+			except Exception:
+				contentEncoding = None
+
+			gzipEnabled = (contentEncoding == 'gzip') or (contentType and contentType.mediaType.startswith('gzip'))
+			d = self._callInstance.processQuery(self.query, gzipEnabled)
 			d.addCallback(processResult)
 			return d
 
@@ -651,57 +708,59 @@ class MultiprocessWorkerOpsiJsonRpc(WorkerOpsiJsonRpc):
 
 
 class WorkerOpsiJsonInterface(WorkerOpsiJsonRpc):
-	def __init__(self, service, request, resource):
-		WorkerOpsiJsonRpc.__init__(self, service, request, resource)
-
+	"""
+	Worker responsible for creating the human-usable interface page.
+	"""
 	def _generateResponse(self, result):
 		logger.info(u"Creating interface page")
 
-		javascript  = u"var currentParams = new Array();\n"
-		javascript += u"var currentMethod = null;\n"
+		javascript = [
+			u"var currentParams = new Array();",
+			u"var currentMethod = null;"
+		]
 		currentMethod = u''
 		if self._rpcs:
 			currentMethod = self._rpcs[0].getMethodName()
-			javascript += u"currentMethod = '%s';\n" % currentMethod
-			for i in range(len(self._rpcs[0].params)):
-				param = self._rpcs[0].params[i]
-				javascript += u"currentParams[%d] = '%s';\n" % (i, toJson(param))
+			javascript.append(u"currentMethod = '%s';" % currentMethod)
+			for (index, param) in enumerate(self._rpcs[0].params):
+				javascript.append(u"currentParams[%d] = '%s';" % (index, toJson(param)))
 
-		selectMethod = u''
+		selectMethod = []
 		for method in self._callInterface:
-			javascript += u"parameters['%s'] = new Array();\n" % (method['name'])
-			for param in range(len(method['params'])):
-				javascript += u"parameters['%s'][%s]='%s';\n" % (method['name'], param, method['params'][param])
-			selected = u''
-			if (method['name'] == currentMethod):
-				selected = u' selected="selected"'
-			selectMethod += u'<option%s>%s</option>' % (selected, method['name'])
+			methodName = method['name']
+			javascript.append(u"parameters['%s'] = new Array();" % methodName)
+			for (index, param) in enumerate(method['params']):
+				javascript.append(u"parameters['%s'][%s]='%s';" % (methodName, index, param))
 
-		resultDiv = u'<div id="result">'
+			selected = u''
+			if method['name'] == currentMethod:
+				selected = u' selected="selected"'
+			selectMethod.append(u'<option%s>%s</option>' % (selected, method['name']))
+
+		def wrapInDiv(obj):
+			return u'<div class="json">{0}</div>'.format(obj)
+
+		results = [u'<div id="result">']
 		if isinstance(result, failure.Failure):
 			error = u'Unknown error'
 			try:
 				result.raiseException()
-			except Exception as e:
-				error = {'class': e.__class__.__name__, 'message': unicode(e)}
+			except Exception as err:
+				error = {'class': err.__class__.__name__, 'message': unicode(err)}
 				error = toJson({"id": None, "result": None, "error": error})
-			resultDiv += u'<div class="json">'
-			resultDiv += objectToHtml(error)
-			resultDiv += u'</div>'
+			results.append(wrapInDiv(objectToHtml(error)))
 		else:
 			for rpc in self._rpcs:
-				resultDiv += u'<div class="json">'
-				resultDiv += objectToHtml(serialize(rpc.getResponse()))
-				resultDiv += u'</div>'
-		resultDiv += u'</div>'
+				results.append(wrapInDiv(objectToHtml(serialize(rpc.getResponse()))))
+		results.append(u'</div>')
 
 		html = interfacePage % {
-			'path':          self.path,
-			'title':         u'opsi interface page',
-			'javascript':    javascript,
-			'select_path':   u'<option selected="selected">%s</option>' % self.path,
-			'select_method': selectMethod,
-			'result':        resultDiv
+			'path': self.path,
+			'title': u'opsi interface page',
+			'javascript': u'\n'.join(javascript),
+			'select_path': u'<option selected="selected">%s</option>' % self.path,
+			'select_method': u''.join(selectMethod),
+			'result': u''.join(results),
 		}
 
 		if not isinstance(result, http.Response):
@@ -716,11 +775,9 @@ class WorkerOpsiJsonInterface(WorkerOpsiJsonRpc):
 
 
 class WorkerOpsiDAV(WorkerOpsi):
-	def __init__(self, service, request, resource):
-		WorkerOpsi.__init__(self, service, request, resource)
-
 	def process(self):
-		logger.debug(u"Worker %s started processing" % self)
+		logger.debug(u"Worker {0} started processing", self)
+
 		deferred = defer.Deferred()
 		if self.resource._authRequired:
 			deferred.addCallback(self._getSession)
@@ -734,11 +791,9 @@ class WorkerOpsiDAV(WorkerOpsi):
 		return deferred
 
 	def _setResponse(self, result):
-		logger.debug(u"Client requests DAV operation: %s" % self.request)
+		logger.debug(u"Client requests DAV operation: {0}", self.request)
 		if not self.resource._authRequired and self.request.method not in ('GET', 'PROPFIND', 'OPTIONS', 'USERINFO', 'HEAD'):
-			logger.critical(u"Method '%s' not allowed (read only)" % self.request.method)
-			return http.Response(
-				code	= responsecode.FORBIDDEN,
-				stream	= "Readonly!" )
+			logger.critical(u"Method '{0}' not allowed (read only)", self.request.method)
+			return http.Response(code=responsecode.FORBIDDEN, stream="Readonly!")
 
 		return self.resource.renderHTTP_super(self.request, self)
