@@ -1,34 +1,31 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+
+# This module is part of the desktop management solution opsi
+# (open pc server integration) http://www.opsi.org
+
+# Copyright (C) 2006-2016 uib GmbH <info@uib.de>
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-opsi python library - File.Opsi
+Utilites to handle files specific to opsi.
 
-This module is part of the desktop management solution opsi
-(open pc server integration) http://www.opsi.org
-
-Copyright (C) 2006-2015 uib GmbH
-
-http://www.uib.de/
-
-All rights reserved.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 :copyright: uib GmbH <info@uib.de>
 :author: Jan Schneider <j.schneider@uib.de>
 :author: Niko Wenselowski <n.wenselowski@uib.de>
-:license: GNU General Public License version 2
+:license: GNU Affero General Public License version 3
 """
 
 import bz2
@@ -46,31 +43,26 @@ from contextlib import closing
 from hashlib import sha1
 from subprocess import Popen, PIPE, STDOUT
 
-if os.name == 'posix':
-	import fcntl
-	import grp
-	import pwd
-	from OPSI.System.Posix import SysInfo
-elif os.name == 'nt':
-	import win32con
-	import win32file
-	import pywintypes
-
 import OPSI.System
 from OPSI.Logger import Logger
 from OPSI.Object import BoolProductProperty, LocalbootProduct, NetbootProduct, Product, ProductDependency, ProductProperty, UnicodeProductProperty
 from OPSI.Types import (BackendBadValueError, OpsiBackupBackendNotFound,
 	OpsiBackupFileError, OpsiBackupFileNotFound, forceActionRequest, forceBool,
 	forceDictList, forceFilename, forceHostId, forceInstallationStatus,
-	forceInt, forceList, forceObjectClass, forceObjectClassList,
-	forceOpsiHostKey, forcePackageVersion, forceProductId,
-	forceProductPriority, forceProductPropertyType, forceProductType,
-	forceProductVersion, forceRequirementType, forceUnicode, forceUnicodeList,
-	forceUnicodeLower)
+	forceList, forceObjectClass, forceObjectClassList, forceOpsiHostKey,
+	forcePackageVersion, forceProductId, forceProductPriority,
+	forceProductPropertyType, forceProductType, forceProductVersion,
+	forceRequirementType, forceUnicode, forceUnicodeList, forceUnicodeLower)
 from OPSI.Util.File import ConfigFile, IniFile, TextFile, requiresParsing
 from OPSI.Util import md5sum, toJson, fromJson
 
-__version__ = '4.0.6.39'
+if os.name == 'posix':
+	import fcntl
+	import grp
+	import pwd
+	from OPSI.System.Posix import SysInfo
+
+__version__ = '4.0.7.27'
 
 logger = Logger()
 
@@ -179,8 +171,7 @@ class BackendACLFile(ConfigFile):
 					aclTypeParam = u''
 					aclTypeParamValues = [u'']
 					inAclTypeParamValues = False
-					for i in range(len(aclTypeParams)):
-						c = aclTypeParams[i]
+					for i, c in enumerate(aclTypeParams):
 						if c == '(':
 							if inAclTypeParamValues:
 								raise Exception(u"Bad formatted acl type params '%s'" % aclTypeParams)
@@ -309,25 +300,31 @@ class PackageContentFile(TextFile):
 		for line in self._lines:
 			(type, tmp) = line.strip().split(None, 1)
 			filename = u''
-			for i in range(len(tmp)):
-				if tmp[i] == u"'":
+
+			for i, currentElement in enumerate(tmp):
+				if currentElement == u"'":
 					if i > 0:
-						if tmp[i-1] == u'\\':
+						if tmp[i - 1] == u'\\':
 							filename = filename[:-1] + u"'"
 							continue
 						else:
 							break
 					else:
 						continue
-				filename += tmp[i]
-			(size, target, md5) = (0, u'', '')
-			tmp = tmp[i+2:]
+				filename += currentElement
+
+			size = 0
+			target = u''
+			md5 = ''
+
+			tmp = tmp[i + 2:]
 			if u' ' in tmp:
 				parts = tmp.split(None, 1)
 				tmp = u''
 				size = parts[0]
 				if len(parts) > 1:
 					tmp = parts[1]
+
 			if type == 'f':
 				md5 = tmp
 			elif type == 'l':
@@ -412,10 +409,7 @@ class PackageControlFile(TextFile):
 
 		sectionType = None
 		option = None
-		lineNum = 0
-		for line in self._lines:
-			lineNum += 1
-
+		for lineNum, line in enumerate(self._lines, start=1):
 			if line and line.startswith((';', '#')):
 				# Comment
 				continue
@@ -569,8 +563,9 @@ class PackageControlFile(TextFile):
 		for (sectionType, secs) in self._sections.items():
 			if sectionType == 'changelog':
 				continue
-			for i in range(len(secs)):
-				for (option, value) in secs[i].items():
+
+			for i, currentSection in enumerate(secs):
+				for (option, value) in currentSection.items():
 					if (sectionType == 'product' and option == 'productclasses') or \
 					   (sectionType == 'package' and option == 'depends') or \
 					   (sectionType == 'productproperty' and option == 'default') or \
@@ -1161,6 +1156,10 @@ class OpsiBackupArchive(tarfile.TarFile):
 			for entry in os.listdir(path):
 				self._addContent(os.path.join(path, entry), sub=sub)
 		else:
+			if not os.path.exists(path):
+				logger.info(u"{0} does not exist. Skipping.", path)
+				return
+
 			checksum = sha1()
 
 			with open(path) as f:
@@ -1327,7 +1326,9 @@ class OpsiBackupArchive(tarfile.TarFile):
 			if not auto or backend["dispatch"]:
 				if not backend["dispatch"]:
 					logger.warning("Backing up backend %s although it's currently not in use." % backend["name"])
-				self._addContent(backend["config"]['dhcpdConfigFile'], sub=(os.path.dirname(backend["config"]['dhcpdConfigFile']), "BACKENDS/DHCP/%s" % backend["name"]))
+
+				dhcpdConfigFile = backend["config"]['dhcpdConfigFile']
+				self._addContent(dhcpdConfigFile, sub=(os.path.dirname(dhcpdConfigFile), "BACKENDS/DHCP/%s" % backend["name"]))
 
 	def hasDHCPBackend(self, name=None):
 		return self._hasBackend("DHCP", name=name)
