@@ -1,8 +1,8 @@
-#!/usr/bin/env python
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
 # This file is part of python-opsi.
-# Copyright (C) 2013 uib GmbH <info@uib.de>
+# Copyright (C) 2013-2016 uib GmbH <info@uib.de>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -33,8 +33,6 @@ For more specialised cleanup you should use the corresponding methods.
 :license: GNU Affero General Public License version 3
 """
 
-from __future__ import unicode_literals
-
 import re
 
 import OPSI.Util.Task.ConfigureBackend as backendUtil
@@ -42,9 +40,11 @@ from OPSI.Backend.BackendManager import BackendManager
 from OPSI.Backend.MySQL import MySQL
 from OPSI.Logger import Logger
 from OPSI.Types import forceBool, forceUnicodeLower
+from OPSI.Util import chunk
 from OPSI.Util.File.Opsi import BackendDispatchConfigFile
 
 LOGGER = Logger()
+_CHUNK_SIZE = 500
 
 
 def cleanupBackend(backend=None):
@@ -64,7 +64,7 @@ BackendManager from default paths.
 		dispatchConfig = bdc.parse()
 		for entry in dispatchConfig:
 			(regex, backends) = entry
-			if not re.search(regex, 'backend_createBase'):
+			if not re.search(regex, u'backend_createBase'):
 				continue
 
 			if 'mysql' in backends:
@@ -72,6 +72,8 @@ BackendManager from default paths.
 
 		return False
 
+	LOGGER.debug("Cleaning backend chunk size: {0}".format(_CHUNK_SIZE))
+
 	if backend is None:
 		backend = BackendManager(
 			dispatchConfigFile=u'/etc/opsi/backendManager/dispatch.conf',
@@ -95,16 +97,13 @@ BackendManager from default paths.
 	LOGGER.notice(u"Cleaning up products")
 	cleanUpProducts(backend)
 
-	LOGGER.debug('Getting current depots...')
-	depotIds = [depot.id for depot in backend.host_getObjects(type=["OpsiConfigserver", "OpsiDepotserver"])]
-	LOGGER.debug('Depots are: {0}'.format(depotIds))
+	LOGGER.debug(u'Getting current depots...')
+	depotIds = set(depot.id for depot in backend.host_getObjects(type=["OpsiConfigserver", "OpsiDepotserver"]))  # pylint: disable=maybe-no-member
+	LOGGER.debug(u'Depots are: {0}'.format(depotIds))
 
-	LOGGER.debug('Getting current products...')
-	productIdents = []
-	for product in backend.product_getObjects():
-		if not product.getIdent(returnType='unicode') in productIdents:
-			productIdents.append(product.getIdent(returnType='unicode'))
-	LOGGER.debug('Product idents are: {0}'.format(productIdents))
+	LOGGER.debug(u'Getting current products...')
+	productIdents = set(product.getIdent(returnType='unicode') for product in backend.product_getObjects())
+	LOGGER.debug(u'Product idents are: {0}'.format(productIdents))
 
 	LOGGER.notice(u"Cleaning up product on depots")
 	cleanUpProductOnDepots(backend, depotIds, productIdents)
@@ -113,48 +112,55 @@ BackendManager from default paths.
 	cleanUpProductOnClients(backend)
 
 	LOGGER.notice(u"Cleaning up product properties")
-	productPropertyIdents = []
+	productPropertyIdents = set()
 	deleteProductProperties = []
 	productPropertiesToCleanup = {}
-	for productProperty in backend.productProperty_getObjects():
+	for productProperty in backend.productProperty_getObjects():  # pylint: disable=maybe-no-member
 		productIdent = u"%s;%s;%s" % (productProperty.productId, productProperty.productVersion, productProperty.packageVersion)
 		if not productProperty.editable and productProperty.possibleValues:
 			productPropertyIdent = u"%s;%s" % (productIdent, productProperty.propertyId)
 			productPropertiesToCleanup[productPropertyIdent] = productProperty
-		if not productIdent in productIdents:
+
+		if productIdent not in productIdents:
 			LOGGER.info(u"Marking productProperty %s of non existent product '%s' for deletion" % (productProperty, productIdent))
 			deleteProductProperties.append(productProperty)
 		else:
 			productPropertyIdent = u'%s;%s' % (productProperty.productId, productProperty.propertyId)
-			if not productPropertyIdent in productPropertyIdents:
-				productPropertyIdents.append(productPropertyIdent)
+			productPropertyIdents.add(productPropertyIdent)
+
 	if deleteProductProperties:
-		backend.productProperty_deleteObjects(deleteProductProperties)
+		for productProperties in chunk(deleteProductProperties, _CHUNK_SIZE):
+			LOGGER.debug(u"Deleting product properties: {0!r}".format(productProperties))
+			backend.productProperty_deleteObjects(productProperties)  # pylint: disable=maybe-no-member
 
 	LOGGER.notice(u"Cleaning up product property states")
 	deleteProductPropertyStates = []
-	for productPropertyState in backend.productPropertyState_getObjects():
+	for productPropertyState in backend.productPropertyState_getObjects():  # pylint: disable=maybe-no-member
 		productPropertyIdent = u'%s;%s' % (productPropertyState.productId, productPropertyState.propertyId)
-		if not productPropertyIdent in productPropertyIdents:
+		if productPropertyIdent not in productPropertyIdents:
 			LOGGER.info(u"Marking productPropertyState %s of non existent productProperty '%s' for deletion" % (productPropertyState, productPropertyIdent))
 			deleteProductPropertyStates.append(productPropertyState)
-	if deleteProductPropertyStates:
-		backend.productPropertyState_deleteObjects(deleteProductPropertyStates)
 
-	for depot in backend.host_getObjects(type='OpsiDepotserver'):
-		objectIds = [depot.id]
-		for clientToDepot in backend.configState_getClientToDepotserver(depotIds=depot.id):
-			if not clientToDepot['clientId'] in objectIds:
-				objectIds.append(clientToDepot['clientId'])
+	if deleteProductPropertyStates:
+		for productPropertyStates in chunk(deleteProductPropertyStates, _CHUNK_SIZE):
+			LOGGER.debug(u"Deleting product property states: {0!r}".format(productPropertyStates))
+			backend.productPropertyState_deleteObjects(productPropertyStates)  # pylint: disable=maybe-no-member
+
+	for depot in backend.host_getObjects(type='OpsiDepotserver'):  # pylint: disable=maybe-no-member
+		objectIds = set(ClientToDepot['clientId'] for ClientToDepot in backend.configState_getClientToDepotserver(depotIds=depot.id))
+		objectIds.add(depot.id)
+
 		productOnDepotIdents = {}
-		for productOnDepot in backend.productOnDepot_getObjects(depotId=depot.id):
+		for productOnDepot in backend.productOnDepot_getObjects(depotId=depot.id):  # pylint: disable=maybe-no-member
 			productIdent = u"%s;%s;%s" % (productOnDepot.productId, productOnDepot.productVersion, productOnDepot.packageVersion)
 			productOnDepotIdents[productOnDepot.productId] = productIdent
+
 		if not productOnDepotIdents:
 			continue
+
 		deleteProductPropertyStates = []
 		updateProductPropertyStates = []
-		for productPropertyState in backend.productPropertyState_getObjects(
+		for productPropertyState in backend.productPropertyState_getObjects(  # pylint: disable=maybe-no-member
 				objectId=objectIds,
 				productId=productOnDepotIdents.keys(),
 				propertyId=[]):
@@ -173,12 +179,12 @@ BackendManager from default paths.
 				if value in productProperty.possibleValues:
 					newValues.append(value)
 					continue
-				if productProperty.getType() == 'BoolProductProperty' and forceBool(value) in productProperty.possibleValues:
+				if productProperty.getType() == u'BoolProductProperty' and forceBool(value) in productProperty.possibleValues:
 					newValues.append(forceBool(value))
 					changedValues.append(value)
 					changed = True
 					continue
-				if productProperty.getType() == 'UnicodeProductProperty':
+				if productProperty.getType() == u'UnicodeProductProperty':
 					newValue = None
 					for possibleValue in productProperty.possibleValues:
 						if forceUnicodeLower(possibleValue) == forceUnicodeLower(value):
@@ -199,10 +205,18 @@ BackendManager from default paths.
 					productPropertyState.setValues(newValues)
 					LOGGER.info(u"Marking productPropertyState %s for update: values not in possible values: %s, values corrected: %s" % (productPropertyState, removeValues, changedValues))
 					updateProductPropertyStates.append(productPropertyState)
+
 		if deleteProductPropertyStates:
-			backend.productPropertyState_deleteObjects(deleteProductPropertyStates)
+			for productPropertyStates in chunk(deleteProductPropertyStates, _CHUNK_SIZE):
+				LOGGER.debug(u"Deleting product property states: {0!r}".format(productPropertyStates))
+				backend.productPropertyState_deleteObjects(productPropertyStates)  # pylint: disable=maybe-no-member
+			del deleteProductPropertyStates
+
 		if updateProductPropertyStates:
-			backend.productPropertyState_updateObjects(updateProductPropertyStates)
+			for productPropertyStates in chunk(updateProductPropertyStates, _CHUNK_SIZE):
+				LOGGER.debug(u"Updating product property states: {0!r}".format(productPropertyStates))
+				backend.productPropertyState_updateObjects(productPropertyStates)  # pylint: disable=maybe-no-member
+			del updateProductPropertyStates
 
 	LOGGER.notice(u"Cleaning up config states")
 	cleanUpConfigStates(backend)
@@ -228,12 +242,12 @@ used MySQL backend.
 	LOGGER.info(u"Current mysql backend config: %s" % config)
 
 	LOGGER.notice(
-		"Connection to database '{database}' on '{address}' as user "
-		"'{username}'".format(**config)
+		u"Connection to database '{database}' on '{address}' as user "
+		u"'{username}'".format(**config)
 	)
 	mysql = MySQL(**config)
 
-	LOGGER.notice("Cleaning up defaultValues in productProperties")
+	LOGGER.notice(u"Cleaning up defaultValues in productProperties")
 	deleteIds = []
 	found = []
 	for res in mysql.getSet("SELECT * FROM PRODUCT_PROPERTY_VALUE WHERE isDefault like '1'"):
@@ -247,7 +261,7 @@ used MySQL backend.
 				deleteIds.append(res['product_property_id'])
 
 	for ID in deleteIds:
-		LOGGER.notice("Deleting PropertyValue id: {0}".format(ID))
+		LOGGER.notice(u"Deleting PropertyValue id: {0}".format(ID))
 		mysql.execute("DELETE FROM `PRODUCT_PROPERTY_VALUE` where "
 			"`product_property_id` = '{0}'".format(ID)
 		)
@@ -262,15 +276,14 @@ def cleanUpGroups(backend):
 	:type backend: OPSI.Backend.Backend
 	"""
 	updatedGroups = []
-	groupIds = []
 	groups = backend.group_getObjects(type='HostGroup')
-	for group in groups:
-		groupIds.append(group.id)
+	groupIds = set(group.id for group in groups)
+
 	for group in groups:
 		if group.getParentGroupId() and group.getParentGroupId() not in groupIds:
 			LOGGER.info(
-				"Removing parent group id '{parentGroupId}' from group "
-				"'{groupId}' because parent group does not exist".format(
+				u"Removing parent group id '{parentGroupId}' from group "
+				u"'{groupId}' because parent group does not exist".format(
 					parentGroupId=group.parentGroupId,
 					groupId=group.id
 				)
@@ -279,7 +292,9 @@ def cleanUpGroups(backend):
 			updatedGroups.append(group)
 
 	if updatedGroups:
-		backend.group_createObjects(updatedGroups)
+		for group in chunk(updatedGroups, _CHUNK_SIZE):
+			LOGGER.debug(u"Updating groups: {0!r}".format(group))
+			backend.group_createObjects(group)
 
 
 def cleanUpProducts(backend):
@@ -289,26 +304,24 @@ def cleanUpProducts(backend):
 	:param backend: The backend where the data should be cleaned.
 	:type backend: OPSI.Backend.Backend
 	"""
-	productIds = []
-	productIdents = []
-
+	productIdents = set()
 	for productOnDepot in backend.productOnDepot_getObjects():
-		productIdent = ";".join([productOnDepot.productId,
-			productOnDepot.productVersion, productOnDepot.packageVersion]
-		)
-		if not productIdent in productIdents:
-			productIdents.append(productIdent)
+		productIdent = ";".join((
+			productOnDepot.productId,
+			productOnDepot.productVersion,
+			productOnDepot.packageVersion
+		))
+		productIdents.add(productIdent)
+
 	deleteProducts = []
 	for product in backend.product_getObjects():
-		if not product.getIdent(returnType='unicode') in productIdents:
-			LOGGER.info("Marking unreferenced product {0} for deletion".format(product))
+		if product.getIdent(returnType='unicode') not in productIdents:
+			LOGGER.info(u"Marking unreferenced product {0} for deletion", product)
 			deleteProducts.append(product)
-		else:
-			if not product.id in productIds:
-				productIds.append(product.id)
 
-	if deleteProducts:
-		backend.product_deleteObjects(deleteProducts)
+	for products in chunk(deleteProducts, _CHUNK_SIZE):
+		LOGGER.debug(u"Deleting products: {0!r}", products)
+		backend.product_deleteObjects(products)
 
 
 def cleanUpProductOnDepots(backend, depotIds, existingProductIdents):
@@ -327,24 +340,26 @@ product is not existing anymore.
 	for productOnDepot in backend.productOnDepot_getObjects():
 		productIdent = ";".join([productOnDepot.productId,
 			productOnDepot.productVersion, productOnDepot.packageVersion])
-		if not productOnDepot.depotId in depotIds:
+		if productOnDepot.depotId not in depotIds:
 			LOGGER.info(
-				"Marking product on depot {poc} for deletion, because "
-				"opsiDepot-Server '{depotId}' not found".format(
+				u"Marking product on depot {poc} for deletion, because "
+				u"opsiDepot-Server '{depotId}' not found".format(
 					poc=productOnDepot,
 					depotId=productOnDepot.depotId
 				)
 			)
 			deleteProductOnDepots.append(productOnDepot)
-		elif not productIdent in existingProductIdents:
+		elif productIdent not in existingProductIdents:
 			LOGGER.info(
-				"Marking product on depot {0} with missing product reference "
-				"for deletion".format(productOnDepot)
+				u"Marking product on depot {0} with missing product reference "
+				u"for deletion".format(productOnDepot)
 			)
 			deleteProductOnDepots.append(productOnDepot)
 
 	if deleteProductOnDepots:
-		backend.productOnDepot_deleteObjects(deleteProductOnDepots)
+		for productOnDepots in chunk(deleteProductOnDepots, _CHUNK_SIZE):
+			LOGGER.debug(u"Deleting products on depots: {0!r}".format(productOnDepots))
+			backend.productOnDepot_deleteObjects(productOnDepots)
 
 
 def cleanUpProductOnClients(backend):
@@ -356,42 +371,42 @@ is either *not_installed* without an action request set.
 	:type backend: OPSI.Backend.Backend
 	"""
 	deleteProductOnClients = []
-	clientIds = []
-	for client in backend.host_getObjects(type=["OpsiClient"]):
-		clientIds.append(client.id)
+	clientIds = set(client.id for client in backend.host_getObjects(type=["OpsiClient"]))
+
 	for productOnClient in backend.productOnClient_getObjects():
 		if productOnClient.clientId not in clientIds:
 			LOGGER.info(
-				"Marking productOnClient {0} for deletion, client "
-				"doesn't exists".format(productOnClient)#
+				u"Marking productOnClient {0} for deletion, client "
+				u"doesn't exists".format(productOnClient)
 			)
 			deleteProductOnClients.append(productOnClient)
-		elif (productOnClient.installationStatus == 'not_installed'
-				and productOnClient.actionRequest == 'none'):
+		elif (productOnClient.installationStatus == u'not_installed'
+				and productOnClient.actionRequest == u'none'):
 			LOGGER.info(
-				"Marking productOnClient {0} for "
-				"deletion".format(productOnClient)
+				u"Marking productOnClient {0} for "
+				u"deletion".format(productOnClient)
 			)
 			deleteProductOnClients.append(productOnClient)
 
 	if deleteProductOnClients:
-		backend.productOnClient_deleteObjects(deleteProductOnClients)
+		for productOnClients in chunk(deleteProductOnClients, _CHUNK_SIZE):
+			LOGGER.debug(u"Deleting products on clients: {0!r}".format(productOnClients))
+			backend.productOnClient_deleteObjects(productOnClients)
 
 	deleteProductOnClients = []
-	productIds = []
-	for product in backend.product_getObjects():
-		if not product.getId() in productIds:
-			productIds.append(product.getId())
+	productIds = set(product.getId() for product in backend.product_getObjects())
 	for productOnClient in backend.productOnClient_getObjects():
-		if not productOnClient.productId in productIds:
+		if productOnClient.productId not in productIds:
 			LOGGER.info(
-				"Marking productOnClient {0} for "
-				"deletion".format(productOnClient)
+				u"Marking productOnClient {0} for "
+				u"deletion".format(productOnClient)
 			)
 			deleteProductOnClients.append(productOnClient)
 
 	if deleteProductOnClients:
-		backend.productOnClient_deleteObjects(deleteProductOnClients)
+		for productOnClients in chunk(deleteProductOnClients, _CHUNK_SIZE):
+			LOGGER.debug(u"Deleting products on clients: {0!r}".format(productOnClients))
+			backend.productOnClient_deleteObjects(productOnClients)
 
 
 def cleanUpConfigStates(backend):
@@ -402,12 +417,13 @@ def cleanUpConfigStates(backend):
 	:type backend: OPSI.Backend.Backend
 	"""
 	deleteConfigStates = []
-	configIds = backend.config_getIdents()
+	configIds = set(backend.config_getIdents())
+
 	for configState in backend.configState_getObjects():
-		if not configState.configId in configIds:
+		if configState.configId not in configIds:
 			LOGGER.info(
-				"Marking configState {configState} of non existent config "
-				"'{config}' for deletion".format(
+				u"Marking configState {configState} of non existent config "
+				u"'{config}' for deletion".format(
 					configState=configState,
 					config=configState.configId
 				)
@@ -415,7 +431,9 @@ def cleanUpConfigStates(backend):
 			deleteConfigStates.append(configState)
 
 	if deleteConfigStates:
-		backend.configState_deleteObjects(deleteConfigStates)
+		for configStates in chunk(deleteConfigStates, _CHUNK_SIZE):
+			LOGGER.debug(u"Deleting config states: {0!r}".format(configStates))
+			backend.configState_deleteObjects(configStates)
 
 
 def cleanUpAuditSoftwares(backend):
@@ -425,16 +443,12 @@ def cleanUpAuditSoftwares(backend):
 	:param backend: The backend where the data should be cleaned.
 	:type backend: OPSI.Backend.Backend
 	"""
-	idents = []
-	for aso in backend.auditSoftwareOnClient_getHashes():
-		ident = '%(name)s;%(version)s;%(subVersion)s;%(language)s;%(architecture)s' % aso
-		if not ident in idents:
-			idents.append(ident)
+	idents = set('%(name)s;%(version)s;%(subVersion)s;%(language)s;%(architecture)s' % aso for aso in backend.auditSoftwareOnClient_getHashes())
 
 	for aso in backend.auditSoftware_getHashes():
 		ident = '%(name)s;%(version)s;%(subVersion)s;%(language)s;%(architecture)s' % aso
-		if not ident in idents:
-			LOGGER.info(u"Deleting unreferenced audit software '%s'" % ident)
+		if ident not in idents:
+			LOGGER.info(u"Deleting unreferenced audit software {0!r}".format(ident))
 			backend.auditSoftware_delete(aso['name'], aso['version'],
 				aso['subVersion'], aso['language'], aso['architecture']
 			)
@@ -447,16 +461,12 @@ def cleanUpAuditSoftwareOnClients(backend):
 	:param backend: The backend where the data should be cleaned.
 	:type backend: OPSI.Backend.Backend
 	"""
-	idents = []
-	for aso in backend.auditSoftware_getHashes():
-		ident = '%(name)s;%(version)s;%(subVersion)s;%(language)s;%(architecture)s' % aso
-		if not ident in idents:
-			idents.append(ident)
+	idents = set('%(name)s;%(version)s;%(subVersion)s;%(language)s;%(architecture)s' % aso for aso in backend.auditSoftware_getHashes())
 
 	for aso in backend.auditSoftwareOnClient_getHashes():
 		ident = '%(name)s;%(version)s;%(subVersion)s;%(language)s;%(architecture)s' % aso
-		if not ident in idents:
-			LOGGER.info(u"Deleting audit software on client '%s'" % ident)
+		if ident not in idents:
+			LOGGER.info(u"Deleting audit software on client {0!r}".format(ident))
 			backend.auditSoftwareOnClient_delete(aso['name'], aso['version'],
 				aso['subVersion'], aso['language'], aso['architecture'],
 				aso['clientId']
